import * as moment from 'moment';
import {
  AnswerQuestionStateViewModel,
  AnswerResponseQuestionStateViewModel,
  AnswerResponseStudentQuestionStateViewModel,
  AnswerStudentQuestionStateViewModel,
  QuestionCategoryQuestionStateViewModel,
  QuestionStateViewModel,
  StudentQuestionStateViewModel,
  TeamQuestionStateViewModel
} from 'src/app/shared-app/models';
import { QuestionStateResponse } from './question-state.models';

export class QuestionMapper {
  public static ToQuestionStateViewModel(questionStateResponse: QuestionStateResponse): QuestionStateViewModel {
    let answers: AnswerQuestionStateViewModel[] = [];

    if (questionStateResponse.answersResponse.length > 0) {
      answers = questionStateResponse.answersResponse.map((x) => {
        let answerResponses: AnswerResponseQuestionStateViewModel[] = [];

        if (x.answerResponsesResponse.length > 0) {
          answerResponses = x.answerResponsesResponse.map((y) => {
            const answerResponseStudent = new AnswerResponseStudentQuestionStateViewModel(
              y.studentResponse.uid,
              y.studentResponse.index,
              y.studentResponse.imageUrl
            );

            return new AnswerResponseQuestionStateViewModel(y.uid, y.text, moment(y.createdOn), answerResponseStudent);
          });
        }

        const answerStudent = new AnswerStudentQuestionStateViewModel(
          x.studentResponse.uid,
          x.studentResponse.index,
          x.studentResponse.imageUrl
        );

        return new AnswerQuestionStateViewModel(
          x.uid,
          x.text,
          x.correctAnswer,
          moment(x.createdOn),
          x.upvotesCount,
          answerStudent,
          answerResponses
        );
      });
    }

    let categories: QuestionCategoryQuestionStateViewModel[] = [];

    if (questionStateResponse.categoriesResponse.length > 0) {
      categories = questionStateResponse.categoriesResponse.map((x) => new QuestionCategoryQuestionStateViewModel(x.uid, x.name));
    }

    let team: TeamQuestionStateViewModel | null = null;

    if (questionStateResponse.teamResponse) {
      team = new TeamQuestionStateViewModel(questionStateResponse.teamResponse.uid, questionStateResponse.teamResponse.name);
    }

    const student = new StudentQuestionStateViewModel(
      questionStateResponse.studentResponse.uid,
      questionStateResponse.studentResponse.index,
      questionStateResponse.studentResponse.imageUrl
    );

    return new QuestionStateViewModel(
      questionStateResponse.uid,
      questionStateResponse.title,
      questionStateResponse.text,
      moment(questionStateResponse.createdOn),
      questionStateResponse.views,
      moment(questionStateResponse.lastActiveOn),
      student,
      answers,
      categories,
      team
    );
  }
}
